@@ -1,17 +1,17 @@
-;;; gpt-mode.el --- Mode-specific functionality for gpt.el -*- lexical-binding: t; -*-
+;;; gpt-mode.el --- Mode-specific functionality for gpt.el -*- lexical-binding: t; package-lint-main-file: "gpt.el"; -*-
 
-;; Copyright (C) 2022 Andreas Stuhlmueller
+;; Copyright (C) 2022-2025 Andreas Stuhlmueller
 
 ;; Author: Andreas Stuhlmueller <emacs@stuhlmueller.org>
 ;; Keywords: openai, anthropic, claude, language, copilot, convenience, tools
 ;; URL: https://github.com/stuhlmueller/gpt.el
 ;; License: MIT
 ;; SPDX-License-Identifier: MIT
-;; Package-Requires: ((emacs "25.1"))
 
 ;;; Commentary:
 
 ;; This file contains mode-specific functionality for gpt.el.
+;; This is the pure Elisp version - no Python dependencies.
 
 ;;; Code:
 
@@ -21,11 +21,19 @@
 (declare-function gpt-insert-command "gpt-ui" (command))
 (declare-function gpt-validate-api-key "gpt-core" nil)
 (declare-function gpt-run-buffer "gpt-api" (output-buffer))
-(declare-function gpt-message "gpt-api" (message))
+(declare-function gpt-message "gpt-api" (format-string &rest args))
 (declare-function gpt-update-model-settings "gpt-core" nil)
 (declare-function gpt-create-output-buffer "gpt-ui" (command))
 (declare-function gpt-read-command "gpt-ui" (context-mode use-selection))
-(declare-function gpt-create-prompt-file "gpt-api" (content))
+(declare-function gpt-abort-request "gpt-api" (&optional buffer))
+(declare-function gpt-google--build-url "gpt-google" (backend model &optional stream))
+
+(defcustom gpt-enable-word-wrap t
+  "Whether to enable word wrapping in GPT buffers.
+When non-nil, long lines are wrapped at word boundaries.
+When nil, long lines are truncated (not wrapped)."
+  :type 'boolean
+  :group 'gpt)
 
 (defface gpt-input-face
   '((t (:inherit comint-highlight-prompt)))
@@ -161,21 +169,22 @@ then specific delimiter lines override the content face.")
 (defun gpt-switch-model ()
   "Switch between OpenAI, Anthropic, and Google models."
   (interactive)
-  (let* ((current-model-name (gpt--find-model-name gpt-api-type gpt-model))
+  (let* ((api-type (gpt--current-api-type))
+         (current-model-name (gpt--find-model-name api-type gpt-model))
          (prompt (format "Choose model (current: %s): "
                          (or current-model-name
-                             (format "%s/%s" gpt-api-type gpt-model))))
+                             (format "%s/%s" api-type gpt-model))))
          (choice (completing-read prompt
                                   (mapcar #'car gpt-available-models)
                                   nil t nil nil current-model-name))
          (model-info (cdr (assoc choice gpt-available-models))))
     (if model-info
         (progn
-          (setq gpt-api-type (plist-get model-info :api)
-                gpt-model (plist-get model-info :id))
+          ;; Only set model ID - gpt-update-model-settings automatically syncs API type
+          (setq gpt-model (plist-get model-info :id))
           (gpt-update-model-settings)  ; Update max_tokens and thinking_budget
           (message "Switched to %s model: %s (max_tokens=%s, thinking_budget=%s)"
-                   (symbol-name gpt-api-type) gpt-model
+                   (symbol-name (gpt--current-api-type)) gpt-model
                    gpt-max-tokens gpt-thinking-budget))
       (message "Model selection cancelled."))))
 
@@ -203,34 +212,44 @@ then specific delimiter lines override the content face.")
   (gpt-validate-api-key)
   (let* ((gpt-buffer (current-buffer))
          (buffer-string (buffer-substring-no-properties (point-min) (point-max)))
-         (prompt (concat buffer-string "\n\nUser: Create a title with a maximum of 50 chars for the chat above. Say only the title, nothing else. No quotes.")))
-    (with-temp-buffer
-      (insert prompt)
-      (let ((prompt-file (gpt-create-prompt-file (current-buffer)))
-            (api-key (cond ((eq gpt-api-type 'openai) gpt-openai-key)
-                           ((eq gpt-api-type 'anthropic) gpt-anthropic-key)
-                           ((eq gpt-api-type 'google) gpt-google-key)
-                           (t "NOT SET")))
-            (api-type-str (symbol-name gpt-api-type))
-            (output-buffer (generate-new-buffer " *gpt-title-output*")))
-        (gpt-message "Asking GPT to generate buffer name...")
-        (unwind-protect
-            (progn
-              ;; Use call-process-region to pass API key via stdin (more secure)
-              (with-temp-buffer
-                (insert api-key "\n")
-                (call-process-region (point-min) (point-max)
-                                     gpt-python-path nil output-buffer nil
-                                     gpt-script-path gpt-model gpt-max-tokens
-                                     gpt-temperature api-type-str prompt-file))
-              (let ((generated-title (string-trim (with-current-buffer output-buffer
-                                                    (buffer-string)))))
-                (with-current-buffer gpt-buffer
-                  (rename-buffer (format "*GPT: %s*" generated-title)))))
-          (when (buffer-live-p output-buffer)
-            (kill-buffer output-buffer))
-          (when (file-exists-p prompt-file)
-            (delete-file prompt-file)))))))
+         (prompt-text (concat buffer-string "\n\nUser: Create a title with a maximum of 50 chars for the chat above. Say only the title, nothing else. No quotes.\n\nAssistant: "))
+         (temp-buffer (generate-new-buffer " *gpt-title*")))
+    (gpt-message "Asking GPT to generate buffer name...")
+    (with-current-buffer temp-buffer
+      (insert prompt-text)
+      ;; Use a simple non-streaming request for title generation
+      (require 'gpt-http)
+      (require 'gpt-backend)
+      (let* ((api-type (gpt--current-api-type))
+             (messages (gpt-backend--parse-messages prompt-text))
+             (backend (gpt-get-backend api-type))
+             (options (list :model gpt-model
+                            :max-tokens 100
+                            :temperature 0))
+             (request-data (gpt-backend-request-data backend messages options))
+             (headers (gpt-backend-headers backend))
+             (url (if (eq api-type 'google)
+                      (progn
+                        (require 'gpt-google)
+                        (gpt-google--build-url backend gpt-model nil))
+                    (oref backend url))))
+        (gpt-http--url-request
+         url headers request-data
+         (lambda (response _http-status error-msg)
+           (if error-msg
+               (progn
+                 (gpt-message "Failed to generate title: %s" error-msg)
+                 (when (buffer-live-p temp-buffer)
+                   (kill-buffer temp-buffer)))
+             (let ((content (gpt-backend-parse-response backend response)))
+               (when (buffer-live-p gpt-buffer)
+                 (with-current-buffer gpt-buffer
+                   (let ((title (if (stringp content)
+                                    (string-trim content)
+                                  (string-trim (plist-get content :content)))))
+                     (rename-buffer (format "*GPT: %s*" title)))))
+               (when (buffer-live-p temp-buffer)
+                 (kill-buffer temp-buffer))))))))))
 
 (defun gpt-chat-clipboard ()
   "Run a GPT command using the current clipboard/kill-ring content as context.
@@ -289,11 +308,6 @@ Useful for quickly processing text copied from other applications."
     map)
   "Keymap for GPT mode.")
 
-(defun gpt--enable-word-wrap ()
-  "Ensure GPT buffers wrap long lines by default."
-  (setq-local truncate-lines nil)
-  (setq-local word-wrap t))
-
 (define-derived-mode gpt-mode text-mode "GPT"
   "A mode for displaying the output of GPT commands.
 This mode provides syntax highlighting for GPT conversations and
@@ -307,7 +321,10 @@ integrates with markdown-mode if available."
       (gpt--setup-markdown-features)
     (gpt--setup-basic-features))
 
-  (gpt--enable-word-wrap)
+  ;; Configure word wrapping based on user preference
+  (when gpt-enable-word-wrap
+    (setq-local truncate-lines nil)
+    (setq-local word-wrap t))
   (add-to-invisibility-spec 'gpt-prefix)
   ;; Use the keymap we defined earlier
   (use-local-map gpt-mode-map)
@@ -319,13 +336,7 @@ integrates with markdown-mode if available."
   (interactive)
   (unless (eq major-mode 'gpt-mode)
     (user-error "Not in a gpt output buffer"))
-  (let ((proc (get-buffer-process (current-buffer))))
-    (if (and proc (process-live-p proc))
-        (progn
-          (delete-process proc)
-          (when (fboundp 'gpt--stop-spinner) (gpt--stop-spinner))
-          (message "Killed GPT process"))
-      (message "No running GPT process"))))
+  (gpt-abort-request (current-buffer)))
 
 (declare-function markdown-mode "markdown-mode" nil)
 
@@ -371,10 +382,21 @@ integrates with markdown-mode if available."
 (defvar gpt--spinner-frames ["◐" "◓" "◑" "◒"]
   "Unicode frames used for the GPT mode-line spinner.")
 
-(defvar-local gpt--spinner-timer nil)
-(defvar-local gpt--spinner-index 0)
-(defvar-local gpt--spinner-string "")
-(defvar-local gpt--spinner-active nil)
+(defvar-local gpt--spinner-timer nil
+  "Timer object for updating the mode-line spinner animation.
+Set to nil when spinner is inactive or when timer is cancelled.")
+
+(defvar-local gpt--spinner-index 0
+  "Current frame index in the spinner animation sequence.
+Cycles through indices 0 to (length gpt--spinner-frames) - 1.")
+
+(defvar-local gpt--spinner-string ""
+  "Current spinner character string displayed in the mode line.
+Updated by the spinner timer to create animation effect.")
+
+(defvar-local gpt--spinner-active nil
+  "Non-nil when the spinner is actively animating in the mode line.
+Set by `gpt--start-spinner' and cleared by `gpt--stop-spinner'.")
 
 (defun gpt--spinner-mode-line ()
   "Return the spinner string for the mode line."
